@@ -7,6 +7,10 @@ const uuid = require('uuid');
 
 const { WebSocketServer } = require('../..');
 
+function onSocketError(err) {
+  console.error(err);
+}
+
 const app = express();
 const map = new Map();
 
@@ -59,6 +63,8 @@ const server = http.createServer(app);
 const wss = new WebSocketServer({ clientTracking: false, noServer: true });
 
 server.on('upgrade', function (request, socket, head) {
+  socket.on('error', onSocketError);
+
   console.log('Parsing session from request...');
 
   sessionParser(request, {}, () => {
@@ -70,6 +76,8 @@ server.on('upgrade', function (request, socket, head) {
 
     console.log('Session is parsed!');
 
+    socket.removeListener('error', onSocketError);
+
     wss.handleUpgrade(request, socket, head, function (ws) {
       wss.emit('connection', ws, request);
     });
@@ -81,6 +89,8 @@ wss.on('connection', function (ws, request) {
 
   map.set(userId, ws);
 
+  ws.on('error', console.error);
+
   ws.on('message', function (message) {
     //
     // Here we can now use session parameters.
