@@ -2,20 +2,20 @@
 
 const assert = require('assert');
 const crypto = require('crypto');
+const EventEmitter = require('events');
 
 const PerMessageDeflate = require('../lib/permessage-deflate');
-const constants = require('../lib/constants');
 const Receiver = require('../lib/receiver');
 const Sender = require('../lib/sender');
-
-const kStatusCode = constants.kStatusCode;
+const { EMPTY_BUFFER, hasBlob, kStatusCode } = require('../lib/constants');
 
 describe('Receiver', () => {
   it('parses an unmasked text message', (done) => {
     const receiver = new Receiver();
 
-    receiver.on('message', (data) => {
-      assert.strictEqual(data, 'Hello');
+    receiver.on('message', (data, isBinary) => {
+      assert.deepStrictEqual(data, Buffer.from('Hello'));
+      assert.ok(!isBinary);
       done();
     });
 
@@ -27,7 +27,7 @@ describe('Receiver', () => {
 
     receiver.on('conclude', (code, data) => {
       assert.strictEqual(code, 1005);
-      assert.strictEqual(data, '');
+      assert.strictEqual(data, EMPTY_BUFFER);
       done();
     });
 
@@ -39,7 +39,7 @@ describe('Receiver', () => {
 
     receiver.on('conclude', (code, data) => {
       assert.strictEqual(code, 1000);
-      assert.strictEqual(data, 'DONE');
+      assert.deepStrictEqual(data, Buffer.from('DONE'));
       done();
     });
 
@@ -48,10 +48,11 @@ describe('Receiver', () => {
   });
 
   it('parses a masked text message', (done) => {
-    const receiver = new Receiver(undefined, {}, true);
+    const receiver = new Receiver({ isServer: true });
 
-    receiver.on('message', (data) => {
-      assert.strictEqual(data, '5:::{"name":"echo"}');
+    receiver.on('message', (data, isBinary) => {
+      assert.deepStrictEqual(data, Buffer.from('5:::{"name":"echo"}'));
+      assert.ok(!isBinary);
       done();
     });
 
@@ -61,21 +62,22 @@ describe('Receiver', () => {
   });
 
   it('parses a masked text message longer than 125 B', (done) => {
-    const receiver = new Receiver(undefined, {}, true);
-    const msg = 'A'.repeat(200);
+    const receiver = new Receiver({ isServer: true });
+    const msg = Buffer.from('A'.repeat(200));
 
-    const list = Sender.frame(Buffer.from(msg), {
+    const list = Sender.frame(msg, {
       fin: true,
       rsv1: false,
       opcode: 0x01,
       mask: true,
-      readOnly: false
+      readOnly: true
     });
 
     const frame = Buffer.concat(list);
 
-    receiver.on('message', (data) => {
-      assert.strictEqual(data, msg);
+    receiver.on('message', (data, isBinary) => {
+      assert.deepStrictEqual(data, msg);
+      assert.ok(!isBinary);
       done();
     });
 
@@ -84,21 +86,22 @@ describe('Receiver', () => {
   });
 
   it('parses a really long masked text message', (done) => {
-    const receiver = new Receiver(undefined, {}, true);
-    const msg = 'A'.repeat(64 * 1024);
+    const receiver = new Receiver({ isServer: true });
+    const msg = Buffer.from('A'.repeat(64 * 1024));
 
-    const list = Sender.frame(Buffer.from(msg), {
+    const list = Sender.frame(msg, {
       fin: true,
       rsv1: false,
       opcode: 0x01,
       mask: true,
-      readOnly: false
+      readOnly: true
     });
 
     const frame = Buffer.concat(list);
 
-    receiver.on('message', (data) => {
-      assert.strictEqual(data, msg);
+    receiver.on('message', (data, isBinary) => {
+      assert.deepStrictEqual(data, msg);
+      assert.ok(!isBinary);
       done();
     });
 
@@ -106,31 +109,32 @@ describe('Receiver', () => {
   });
 
   it('parses a 300 B fragmented masked text message', (done) => {
-    const receiver = new Receiver(undefined, {}, true);
-    const msg = 'A'.repeat(300);
+    const receiver = new Receiver({ isServer: true });
+    const msg = Buffer.from('A'.repeat(300));
 
-    const fragment1 = msg.substr(0, 150);
-    const fragment2 = msg.substr(150);
+    const fragment1 = msg.slice(0, 150);
+    const fragment2 = msg.slice(150);
 
-    const options = { rsv1: false, mask: true, readOnly: false };
+    const options = { rsv1: false, mask: true, readOnly: true };
 
     const frame1 = Buffer.concat(
-      Sender.frame(Buffer.from(fragment1), {
+      Sender.frame(fragment1, {
         fin: false,
         opcode: 0x01,
         ...options
       })
     );
     const frame2 = Buffer.concat(
-      Sender.frame(Buffer.from(fragment2), {
+      Sender.frame(fragment2, {
         fin: true,
         opcode: 0x00,
         ...options
       })
     );
 
-    receiver.on('message', (data) => {
-      assert.strictEqual(data, msg);
+    receiver.on('message', (data, isBinary) => {
+      assert.deepStrictEqual(data, msg);
+      assert.ok(!isBinary);
       done();
     });
 
@@ -139,21 +143,21 @@ describe('Receiver', () => {
   });
 
   it('parses a ping message', (done) => {
-    const receiver = new Receiver(undefined, {}, true);
-    const msg = 'Hello';
+    const receiver = new Receiver({ isServer: true });
+    const msg = Buffer.from('Hello');
 
-    const list = Sender.frame(Buffer.from(msg), {
+    const list = Sender.frame(msg, {
       fin: true,
       rsv1: false,
       opcode: 0x09,
       mask: true,
-      readOnly: false
+      readOnly: true
     });
 
     const frame = Buffer.concat(list);
 
     receiver.on('ping', (data) => {
-      assert.strictEqual(data.toString(), msg);
+      assert.deepStrictEqual(data, msg);
       done();
     });
 
@@ -164,7 +168,7 @@ describe('Receiver', () => {
     const receiver = new Receiver();
 
     receiver.on('ping', (data) => {
-      assert.ok(data.equals(Buffer.alloc(0)));
+      assert.strictEqual(data, EMPTY_BUFFER);
       done();
     });
 
@@ -172,31 +176,31 @@ describe('Receiver', () => {
   });
 
   it('parses a 300 B fragmented masked text message with a ping in the middle (1/2)', (done) => {
-    const receiver = new Receiver(undefined, {}, true);
-    const msg = 'A'.repeat(300);
-    const pingMessage = 'Hello';
+    const receiver = new Receiver({ isServer: true });
+    const msg = Buffer.from('A'.repeat(300));
+    const pingMessage = Buffer.from('Hello');
 
-    const fragment1 = msg.substr(0, 150);
-    const fragment2 = msg.substr(150);
+    const fragment1 = msg.slice(0, 150);
+    const fragment2 = msg.slice(150);
 
-    const options = { rsv1: false, mask: true, readOnly: false };
+    const options = { rsv1: false, mask: true, readOnly: true };
 
     const frame1 = Buffer.concat(
-      Sender.frame(Buffer.from(fragment1), {
+      Sender.frame(fragment1, {
         fin: false,
         opcode: 0x01,
         ...options
       })
     );
     const frame2 = Buffer.concat(
-      Sender.frame(Buffer.from(pingMessage), {
+      Sender.frame(pingMessage, {
         fin: true,
         opcode: 0x09,
         ...options
       })
     );
     const frame3 = Buffer.concat(
-      Sender.frame(Buffer.from(fragment2), {
+      Sender.frame(fragment2, {
         fin: true,
         opcode: 0x00,
         ...options
@@ -205,14 +209,15 @@ describe('Receiver', () => {
 
     let gotPing = false;
 
-    receiver.on('message', (data) => {
-      assert.strictEqual(data, msg);
+    receiver.on('message', (data, isBinary) => {
+      assert.deepStrictEqual(data, msg);
+      assert.ok(!isBinary);
       assert.ok(gotPing);
       done();
     });
     receiver.on('ping', (data) => {
       gotPing = true;
-      assert.strictEqual(data.toString(), pingMessage);
+      assert.ok(data.equals(pingMessage));
     });
 
     receiver.write(frame1);
@@ -221,12 +226,12 @@ describe('Receiver', () => {
   });
 
   it('parses a 300 B fragmented masked text message with a ping in the middle (2/2)', (done) => {
-    const receiver = new Receiver(undefined, {}, true);
-    const msg = 'A'.repeat(300);
-    const pingMessage = 'Hello';
+    const receiver = new Receiver({ isServer: true });
+    const msg = Buffer.from('A'.repeat(300));
+    const pingMessage = Buffer.from('Hello');
 
-    const fragment1 = msg.substr(0, 150);
-    const fragment2 = msg.substr(150);
+    const fragment1 = msg.slice(0, 150);
+    const fragment2 = msg.slice(150);
 
     const options = { rsv1: false, mask: true, readOnly: false };
 
@@ -264,14 +269,15 @@ describe('Receiver', () => {
 
     let gotPing = false;
 
-    receiver.on('message', (data) => {
-      assert.strictEqual(data, msg);
+    receiver.on('message', (data, isBinary) => {
+      assert.deepStrictEqual(data, msg);
+      assert.ok(!isBinary);
       assert.ok(gotPing);
       done();
     });
     receiver.on('ping', (data) => {
       gotPing = true;
-      assert.strictEqual(data.toString(), pingMessage);
+      assert.ok(data.equals(pingMessage));
     });
 
     for (let i = 0; i < chunks.length; ++i) {
@@ -280,7 +286,7 @@ describe('Receiver', () => {
   });
 
   it('parses a 100 B masked binary message', (done) => {
-    const receiver = new Receiver(undefined, {}, true);
+    const receiver = new Receiver({ isServer: true });
     const msg = crypto.randomBytes(100);
 
     const list = Sender.frame(msg, {
@@ -293,8 +299,9 @@ describe('Receiver', () => {
 
     const frame = Buffer.concat(list);
 
-    receiver.on('message', (data) => {
-      assert.ok(data.equals(msg));
+    receiver.on('message', (data, isBinary) => {
+      assert.deepStrictEqual(data, msg);
+      assert.ok(isBinary);
       done();
     });
 
@@ -302,7 +309,7 @@ describe('Receiver', () => {
   });
 
   it('parses a 256 B masked binary message', (done) => {
-    const receiver = new Receiver(undefined, {}, true);
+    const receiver = new Receiver({ isServer: true });
     const msg = crypto.randomBytes(256);
 
     const list = Sender.frame(msg, {
@@ -315,8 +322,9 @@ describe('Receiver', () => {
 
     const frame = Buffer.concat(list);
 
-    receiver.on('message', (data) => {
-      assert.ok(data.equals(msg));
+    receiver.on('message', (data, isBinary) => {
+      assert.deepStrictEqual(data, msg);
+      assert.ok(isBinary);
       done();
     });
 
@@ -324,7 +332,7 @@ describe('Receiver', () => {
   });
 
   it('parses a 200 KiB masked binary message', (done) => {
-    const receiver = new Receiver(undefined, {}, true);
+    const receiver = new Receiver({ isServer: true });
     const msg = crypto.randomBytes(200 * 1024);
 
     const list = Sender.frame(msg, {
@@ -337,8 +345,9 @@ describe('Receiver', () => {
 
     const frame = Buffer.concat(list);
 
-    receiver.on('message', (data) => {
-      assert.ok(data.equals(msg));
+    receiver.on('message', (data, isBinary) => {
+      assert.deepStrictEqual(data, msg);
+      assert.ok(isBinary);
       done();
     });
 
@@ -359,8 +368,9 @@ describe('Receiver', () => {
 
     const frame = Buffer.concat(list);
 
-    receiver.on('message', (data) => {
-      assert.ok(data.equals(msg));
+    receiver.on('message', (data, isBinary) => {
+      assert.deepStrictEqual(data, msg);
+      assert.ok(isBinary);
       done();
     });
 
@@ -371,13 +381,16 @@ describe('Receiver', () => {
     const perMessageDeflate = new PerMessageDeflate();
     perMessageDeflate.accept([{}]);
 
-    const receiver = new Receiver(undefined, {
-      'permessage-deflate': perMessageDeflate
+    const receiver = new Receiver({
+      extensions: {
+        'permessage-deflate': perMessageDeflate
+      }
     });
     const buf = Buffer.from('Hello');
 
-    receiver.on('message', (data) => {
-      assert.strictEqual(data, 'Hello');
+    receiver.on('message', (data, isBinary) => {
+      assert.deepStrictEqual(data, buf);
+      assert.ok(!isBinary);
       done();
     });
 
@@ -393,14 +406,17 @@ describe('Receiver', () => {
     const perMessageDeflate = new PerMessageDeflate();
     perMessageDeflate.accept([{}]);
 
-    const receiver = new Receiver(undefined, {
-      'permessage-deflate': perMessageDeflate
+    const receiver = new Receiver({
+      extensions: {
+        'permessage-deflate': perMessageDeflate
+      }
     });
     const buf1 = Buffer.from('foo');
     const buf2 = Buffer.from('bar');
 
-    receiver.on('message', (data) => {
-      assert.strictEqual(data, 'foobar');
+    receiver.on('message', (data, isBinary) => {
+      assert.deepStrictEqual(data, Buffer.concat([buf1, buf2]));
+      assert.ok(!isBinary);
       done();
     });
 
@@ -430,8 +446,9 @@ describe('Receiver', () => {
     const receiver = new Receiver();
     let counter = 0;
 
-    receiver.on('message', (data) => {
-      assert.strictEqual(data, '');
+    receiver.on('message', (data, isBinary) => {
+      assert.strictEqual(data, EMPTY_BUFFER);
+      assert.ok(!isBinary);
       if (++counter === 20000) done();
     });
 
@@ -439,11 +456,12 @@ describe('Receiver', () => {
   });
 
   it('resets `totalPayloadLength` only on final frame (unfragmented)', (done) => {
-    const receiver = new Receiver(undefined, {}, false, 10);
+    const receiver = new Receiver({ maxPayload: 10 });
 
-    receiver.on('message', (data) => {
+    receiver.on('message', (data, isBinary) => {
       assert.strictEqual(receiver._totalPayloadLength, 0);
-      assert.strictEqual(data, 'Hello');
+      assert.deepStrictEqual(data, Buffer.from('Hello'));
+      assert.ok(!isBinary);
       done();
     });
 
@@ -452,11 +470,12 @@ describe('Receiver', () => {
   });
 
   it('resets `totalPayloadLength` only on final frame (fragmented)', (done) => {
-    const receiver = new Receiver(undefined, {}, false, 10);
+    const receiver = new Receiver({ maxPayload: 10 });
 
-    receiver.on('message', (data) => {
+    receiver.on('message', (data, isBinary) => {
       assert.strictEqual(receiver._totalPayloadLength, 0);
-      assert.strictEqual(data, 'Hello');
+      assert.deepStrictEqual(data, Buffer.from('Hello'));
+      assert.ok(!isBinary);
       done();
     });
 
@@ -467,17 +486,18 @@ describe('Receiver', () => {
   });
 
   it('resets `totalPayloadLength` only on final frame (fragmented + ping)', (done) => {
-    const receiver = new Receiver(undefined, {}, false, 10);
+    const receiver = new Receiver({ maxPayload: 10 });
     let data;
 
     receiver.on('ping', (buf) => {
       assert.strictEqual(receiver._totalPayloadLength, 2);
-      data = buf.toString();
+      data = buf;
     });
-    receiver.on('message', (buf) => {
+    receiver.on('message', (buf, isBinary) => {
       assert.strictEqual(receiver._totalPayloadLength, 0);
-      assert.strictEqual(data, '');
-      assert.strictEqual(buf.toString(), 'Hello');
+      assert.deepStrictEqual(data, EMPTY_BUFFER);
+      assert.deepStrictEqual(buf, Buffer.from('Hello'));
+      assert.ok(isBinary);
       done();
     });
 
@@ -491,15 +511,22 @@ describe('Receiver', () => {
     const perMessageDeflate = new PerMessageDeflate();
     perMessageDeflate.accept([{}]);
 
-    const receiver = new Receiver(undefined, {
-      'permessage-deflate': perMessageDeflate
+    const receiver = new Receiver({
+      extensions: {
+        'permessage-deflate': perMessageDeflate
+      }
     });
     const results = [];
     const push = results.push.bind(results);
 
     receiver.on('conclude', push).on('message', push);
     receiver.on('finish', () => {
-      assert.deepStrictEqual(results, ['', 1005, '']);
+      assert.deepStrictEqual(results, [
+        EMPTY_BUFFER,
+        false,
+        1005,
+        EMPTY_BUFFER
+      ]);
       done();
     });
 
@@ -513,6 +540,7 @@ describe('Receiver', () => {
 
     receiver.on('error', (err) => {
       assert.ok(err instanceof RangeError);
+      assert.strictEqual(err.code, 'WS_ERR_UNEXPECTED_RSV_1');
       assert.strictEqual(
         err.message,
         'Invalid WebSocket frame: RSV1 must be clear'
@@ -528,12 +556,15 @@ describe('Receiver', () => {
     const perMessageDeflate = new PerMessageDeflate();
     perMessageDeflate.accept([{}]);
 
-    const receiver = new Receiver(undefined, {
-      'permessage-deflate': perMessageDeflate
+    const receiver = new Receiver({
+      extensions: {
+        'permessage-deflate': perMessageDeflate
+      }
     });
 
     receiver.on('error', (err) => {
       assert.ok(err instanceof RangeError);
+      assert.strictEqual(err.code, 'WS_ERR_UNEXPECTED_RSV_1');
       assert.strictEqual(
         err.message,
         'Invalid WebSocket frame: RSV1 must be clear'
@@ -550,6 +581,7 @@ describe('Receiver', () => {
 
     receiver.on('error', (err) => {
       assert.ok(err instanceof RangeError);
+      assert.strictEqual(err.code, 'WS_ERR_UNEXPECTED_RSV_2_3');
       assert.strictEqual(
         err.message,
         'Invalid WebSocket frame: RSV2 and RSV3 must be clear'
@@ -566,6 +598,7 @@ describe('Receiver', () => {
 
     receiver.on('error', (err) => {
       assert.ok(err instanceof RangeError);
+      assert.strictEqual(err.code, 'WS_ERR_UNEXPECTED_RSV_2_3');
       assert.strictEqual(
         err.message,
         'Invalid WebSocket frame: RSV2 and RSV3 must be clear'
@@ -582,6 +615,7 @@ describe('Receiver', () => {
 
     receiver.on('error', (err) => {
       assert.ok(err instanceof RangeError);
+      assert.strictEqual(err.code, 'WS_ERR_INVALID_OPCODE');
       assert.strictEqual(
         err.message,
         'Invalid WebSocket frame: invalid opcode 0'
@@ -598,6 +632,7 @@ describe('Receiver', () => {
 
     receiver.on('error', (err) => {
       assert.ok(err instanceof RangeError);
+      assert.strictEqual(err.code, 'WS_ERR_INVALID_OPCODE');
       assert.strictEqual(
         err.message,
         'Invalid WebSocket frame: invalid opcode 1'
@@ -615,6 +650,7 @@ describe('Receiver', () => {
 
     receiver.on('error', (err) => {
       assert.ok(err instanceof RangeError);
+      assert.strictEqual(err.code, 'WS_ERR_INVALID_OPCODE');
       assert.strictEqual(
         err.message,
         'Invalid WebSocket frame: invalid opcode 2'
@@ -632,6 +668,7 @@ describe('Receiver', () => {
 
     receiver.on('error', (err) => {
       assert.ok(err instanceof RangeError);
+      assert.strictEqual(err.code, 'WS_ERR_EXPECTED_FIN');
       assert.strictEqual(
         err.message,
         'Invalid WebSocket frame: FIN must be set'
@@ -647,12 +684,15 @@ describe('Receiver', () => {
     const perMessageDeflate = new PerMessageDeflate();
     perMessageDeflate.accept([{}]);
 
-    const receiver = new Receiver(undefined, {
-      'permessage-deflate': perMessageDeflate
+    const receiver = new Receiver({
+      extensions: {
+        'permessage-deflate': perMessageDeflate
+      }
     });
 
     receiver.on('error', (err) => {
       assert.ok(err instanceof RangeError);
+      assert.strictEqual(err.code, 'WS_ERR_UNEXPECTED_RSV_1');
       assert.strictEqual(
         err.message,
         'Invalid WebSocket frame: RSV1 must be clear'
@@ -669,6 +709,7 @@ describe('Receiver', () => {
 
     receiver.on('error', (err) => {
       assert.ok(err instanceof RangeError);
+      assert.strictEqual(err.code, 'WS_ERR_EXPECTED_FIN');
       assert.strictEqual(
         err.message,
         'Invalid WebSocket frame: FIN must be set'
@@ -681,10 +722,11 @@ describe('Receiver', () => {
   });
 
   it('emits an error if a frame has the MASK bit off (server mode)', (done) => {
-    const receiver = new Receiver(undefined, {}, true);
+    const receiver = new Receiver({ isServer: true });
 
     receiver.on('error', (err) => {
       assert.ok(err instanceof RangeError);
+      assert.strictEqual(err.code, 'WS_ERR_EXPECTED_MASK');
       assert.strictEqual(
         err.message,
         'Invalid WebSocket frame: MASK must be set'
@@ -697,10 +739,11 @@ describe('Receiver', () => {
   });
 
   it('emits an error if a frame has the MASK bit on (client mode)', (done) => {
-    const receiver = new Receiver(undefined, {}, false);
+    const receiver = new Receiver();
 
     receiver.on('error', (err) => {
       assert.ok(err instanceof RangeError);
+      assert.strictEqual(err.code, 'WS_ERR_UNEXPECTED_MASK');
       assert.strictEqual(
         err.message,
         'Invalid WebSocket frame: MASK must be clear'
@@ -719,6 +762,7 @@ describe('Receiver', () => {
 
     receiver.on('error', (err) => {
       assert.ok(err instanceof RangeError);
+      assert.strictEqual(err.code, 'WS_ERR_INVALID_CONTROL_PAYLOAD_LENGTH');
       assert.strictEqual(
         err.message,
         'Invalid WebSocket frame: invalid payload length 126'
@@ -735,6 +779,7 @@ describe('Receiver', () => {
 
     receiver.on('error', (err) => {
       assert.ok(err instanceof RangeError);
+      assert.strictEqual(err.code, 'WS_ERR_UNSUPPORTED_DATA_PAYLOAD_LENGTH');
       assert.strictEqual(
         err.message,
         'Unsupported WebSocket frame: payload length > 2^53 - 1'
@@ -756,6 +801,7 @@ describe('Receiver', () => {
 
     receiver.on('error', (err) => {
       assert.ok(err instanceof Error);
+      assert.strictEqual(err.code, 'WS_ERR_INVALID_UTF8');
       assert.strictEqual(
         err.message,
         'Invalid WebSocket frame: invalid UTF-8 sequence'
@@ -771,13 +817,16 @@ describe('Receiver', () => {
     const perMessageDeflate = new PerMessageDeflate();
     perMessageDeflate.accept([{}]);
 
-    const receiver = new Receiver(undefined, {
-      'permessage-deflate': perMessageDeflate
+    const receiver = new Receiver({
+      extensions: {
+        'permessage-deflate': perMessageDeflate
+      }
     });
     const buf = Buffer.from([0xce, 0xba, 0xe1, 0xbd]);
 
     receiver.on('error', (err) => {
       assert.ok(err instanceof Error);
+      assert.strictEqual(err.code, 'WS_ERR_INVALID_UTF8');
       assert.strictEqual(
         err.message,
         'Invalid WebSocket frame: invalid UTF-8 sequence'
@@ -799,6 +848,7 @@ describe('Receiver', () => {
 
     receiver.on('error', (err) => {
       assert.ok(err instanceof RangeError);
+      assert.strictEqual(err.code, 'WS_ERR_INVALID_CONTROL_PAYLOAD_LENGTH');
       assert.strictEqual(
         err.message,
         'Invalid WebSocket frame: invalid payload length 1'
@@ -807,7 +857,7 @@ describe('Receiver', () => {
       done();
     });
 
-    receiver.write(Buffer.from([0x88, 0x01, 0x00]));
+    receiver.write(Buffer.from([0x88, 0x01]));
   });
 
   it('emits an error if a close frame contains an invalid close code', (done) => {
@@ -815,6 +865,7 @@ describe('Receiver', () => {
 
     receiver.on('error', (err) => {
       assert.ok(err instanceof RangeError);
+      assert.strictEqual(err.code, 'WS_ERR_INVALID_CLOSE_CODE');
       assert.strictEqual(
         err.message,
         'Invalid WebSocket frame: invalid status code 0'
@@ -831,6 +882,7 @@ describe('Receiver', () => {
 
     receiver.on('error', (err) => {
       assert.ok(err instanceof Error);
+      assert.strictEqual(err.code, 'WS_ERR_INVALID_UTF8');
       assert.strictEqual(
         err.message,
         'Invalid WebSocket frame: invalid UTF-8 sequence'
@@ -845,7 +897,7 @@ describe('Receiver', () => {
   });
 
   it('emits an error if a frame payload length is bigger than `maxPayload`', (done) => {
-    const receiver = new Receiver(undefined, {}, true, 20 * 1024);
+    const receiver = new Receiver({ isServer: true, maxPayload: 20 * 1024 });
     const msg = crypto.randomBytes(200 * 1024);
 
     const list = Sender.frame(msg, {
@@ -860,6 +912,7 @@ describe('Receiver', () => {
 
     receiver.on('error', (err) => {
       assert.ok(err instanceof RangeError);
+      assert.strictEqual(err.code, 'WS_ERR_UNSUPPORTED_MESSAGE_LENGTH');
       assert.strictEqual(err.message, 'Max payload size exceeded');
       assert.strictEqual(err[kStatusCode], 1009);
       done();
@@ -872,18 +925,16 @@ describe('Receiver', () => {
     const perMessageDeflate = new PerMessageDeflate({}, false, 25);
     perMessageDeflate.accept([{}]);
 
-    const receiver = new Receiver(
-      undefined,
-      {
-        'permessage-deflate': perMessageDeflate
-      },
-      false,
-      25
-    );
+    const receiver = new Receiver({
+      extensions: { 'permessage-deflate': perMessageDeflate },
+      isServer: false,
+      maxPayload: 25
+    });
     const buf = Buffer.from('A'.repeat(50));
 
     receiver.on('error', (err) => {
       assert.ok(err instanceof RangeError);
+      assert.strictEqual(err.code, 'WS_ERR_UNSUPPORTED_MESSAGE_LENGTH');
       assert.strictEqual(err.message, 'Max payload size exceeded');
       assert.strictEqual(err[kStatusCode], 1009);
       done();
@@ -901,18 +952,16 @@ describe('Receiver', () => {
     const perMessageDeflate = new PerMessageDeflate({}, false, 25);
     perMessageDeflate.accept([{}]);
 
-    const receiver = new Receiver(
-      undefined,
-      {
-        'permessage-deflate': perMessageDeflate
-      },
-      false,
-      25
-    );
+    const receiver = new Receiver({
+      extensions: { 'permessage-deflate': perMessageDeflate },
+      isServer: false,
+      maxPayload: 25
+    });
     const buf = Buffer.from('A'.repeat(15));
 
     receiver.on('error', (err) => {
       assert.ok(err instanceof RangeError);
+      assert.strictEqual(err.code, 'WS_ERR_UNSUPPORTED_MESSAGE_LENGTH');
       assert.strictEqual(err.message, 'Max payload size exceeded');
       assert.strictEqual(err[kStatusCode], 1009);
       done();
@@ -942,9 +991,9 @@ describe('Receiver', () => {
       crypto.randomBytes(3)
     ];
 
-    receiver.on('message', (data) => {
-      assert.ok(Buffer.isBuffer(data));
-      assert.ok(data.equals(Buffer.concat(frags)));
+    receiver.on('message', (data, isBinary) => {
+      assert.deepStrictEqual(data, Buffer.concat(frags));
+      assert.ok(isBinary);
       done();
     });
 
@@ -960,17 +1009,17 @@ describe('Receiver', () => {
   });
 
   it("honors the 'arraybuffer' binary type", (done) => {
-    const receiver = new Receiver();
+    const receiver = new Receiver({ binaryType: 'arraybuffer' });
     const frags = [
       crypto.randomBytes(19221),
       crypto.randomBytes(954),
       crypto.randomBytes(623987)
     ];
 
-    receiver._binaryType = 'arraybuffer';
-    receiver.on('message', (data) => {
+    receiver.on('message', (data, isBinary) => {
       assert.ok(data instanceof ArrayBuffer);
-      assert.ok(Buffer.from(data).equals(Buffer.concat(frags)));
+      assert.deepStrictEqual(Buffer.from(data), Buffer.concat(frags));
+      assert.ok(isBinary);
       done();
     });
 
@@ -986,7 +1035,7 @@ describe('Receiver', () => {
   });
 
   it("honors the 'fragments' binary type", (done) => {
-    const receiver = new Receiver();
+    const receiver = new Receiver({ binaryType: 'fragments' });
     const frags = [
       crypto.randomBytes(17),
       crypto.randomBytes(419872),
@@ -995,9 +1044,9 @@ describe('Receiver', () => {
       crypto.randomBytes(1)
     ];
 
-    receiver._binaryType = 'fragments';
-    receiver.on('message', (data) => {
+    receiver.on('message', (data, isBinary) => {
       assert.deepStrictEqual(data, frags);
+      assert.ok(isBinary);
       done();
     });
 
@@ -1011,4 +1060,142 @@ describe('Receiver', () => {
       }).forEach((buf) => receiver.write(buf));
     });
   });
+
+  it("honors the 'blob' binary type", function (done) {
+    if (!hasBlob) return this.skip();
+
+    const receiver = new Receiver({ binaryType: 'blob' });
+    const frags = [
+      crypto.randomBytes(75688),
+      crypto.randomBytes(2688),
+      crypto.randomBytes(46753)
+    ];
+
+    receiver.on('message', (data, isBinary) => {
+      assert.ok(data instanceof Blob);
+      assert.ok(isBinary);
+
+      data
+        .arrayBuffer()
+        .then((arrayBuffer) => {
+          assert.deepStrictEqual(
+            Buffer.from(arrayBuffer),
+            Buffer.concat(frags)
+          );
+
+          done();
+        })
+        .catch(done);
+    });
+
+    frags.forEach((frag, i) => {
+      Sender.frame(frag, {
+        fin: i === frags.length - 1,
+        opcode: i === 0 ? 2 : 0,
+        readOnly: true,
+        mask: false,
+        rsv1: false
+      }).forEach((buf) => receiver.write(buf));
+    });
+  });
+
+  it('honors the `skipUTF8Validation` option (1/2)', (done) => {
+    const receiver = new Receiver({ skipUTF8Validation: true });
+
+    receiver.on('message', (data, isBinary) => {
+      assert.deepStrictEqual(data, Buffer.from([0xf8]));
+      assert.ok(!isBinary);
+      done();
+    });
+
+    receiver.write(Buffer.from([0x81, 0x01, 0xf8]));
+  });
+
+  it('honors the `skipUTF8Validation` option (2/2)', (done) => {
+    const receiver = new Receiver({ skipUTF8Validation: true });
+
+    receiver.on('conclude', (code, data) => {
+      assert.strictEqual(code, 1000);
+      assert.deepStrictEqual(data, Buffer.from([0xf8]));
+      done();
+    });
+
+    receiver.write(Buffer.from([0x88, 0x03, 0x03, 0xe8, 0xf8]));
+  });
+
+  it('honors the `allowSynchronousEvents` option', (done) => {
+    const actual = [];
+    const expected = [
+      '1',
+      '- 1',
+      '-- 1',
+      '2',
+      '- 2',
+      '-- 2',
+      '3',
+      '- 3',
+      '-- 3',
+      '4',
+      '- 4',
+      '-- 4'
+    ];
+
+    function listener(data) {
+      const message = data.toString();
+      actual.push(message);
+
+      // `queueMicrotask()` is not available in Node.js < 11.
+      Promise.resolve().then(() => {
+        actual.push(`- ${message}`);
+
+        Promise.resolve().then(() => {
+          actual.push(`-- ${message}`);
+
+          if (actual.length === 12) {
+            assert.deepStrictEqual(actual, expected);
+            done();
+          }
+        });
+      });
+    }
+
+    const receiver = new Receiver({ allowSynchronousEvents: false });
+
+    receiver.on('message', listener);
+    receiver.on('ping', listener);
+    receiver.on('pong', listener);
+
+    receiver.write(Buffer.from('8101318901328a0133820134', 'hex'));
+  });
+
+  it('does not swallow errors thrown from event handlers', (done) => {
+    const receiver = new Receiver();
+    let count = 0;
+
+    receiver.on('message', () => {
+      if (++count === 2) {
+        throw new Error('Oops');
+      }
+    });
+
+    assert.strictEqual(
+      process.listenerCount('uncaughtException'),
+      EventEmitter.usingDomains ? 2 : 1
+    );
+
+    const listener = process.listeners('uncaughtException').pop();
+
+    process.removeListener('uncaughtException', listener);
+    process.once('uncaughtException', (err) => {
+      assert.ok(err instanceof Error);
+      assert.strictEqual(err.message, 'Oops');
+
+      process.on('uncaughtException', listener);
+      done();
+    });
+
+    setImmediate(() => {
+      receiver.write(Buffer.from('82008200', 'hex'));
+    });
+  });
 });
